////////////////////////////////////////////////////////////////////////////////
// awe_sequencer.h - AWEWavePlayer version 0.3 by Jeroen Witteman
////////////////////////////////////////////////////////////////////////////////
//
// Written by:    Jeroen Witteman
// Email adress:  jwittema@wi.leidenuniv.nl
// WWW Homepage:  http://www.wi.leidenuniv.nl/home/jwittema/
//
// copyright (c) 1998 by Jeroen Witteman, all rights reserved.
// THIS SOFTWARE COMES WITH ABSOLUTELY NO WARRANTY! USE AT YOUR OWN RISK!
////////////////////////////////////////////////////////////////////////////////

#ifndef INC_AWE_SEQUENCER
#define INC_AWE_SEQUENCER

// defines
#define MonoSampleSize (StereoSampleSize/2)
#define PANLEFT   0      // the left panning value of awe_voice_info
#define PANRIGHT  127    // the right panning value of awe_voice_info
#define REPLACE   true
#define NOREPLACE false
#define IDOS      64     // the offset sample_id from left <-> right sample
#define ROOT_KEY  22     // root key

#define MIN_NROFSAMPLES 2
#define MAX_NROFSAMPLES 64
#define MIN_CHORUS 0
#define MAX_CHORUS 127
#define MIN_REVERB 0
#define MAX_REVERB 127

#define MIN_CHORUSM 0
#define MAX_CHORUSM 7
#define MIN_REVERBM 0
#define MAX_REVERBM 7

#define MIN_BASS 0
#define MAX_BASS 11
#define MIN_TREBLE 0
#define MAX_TREBLE 11
#define MIN_VOLUME 0
#define MAX_VOLUME 127

class AWE_Sequencer {
  public:
    void Initialize();
    void Shutdown();

    void StartPlaying();
    void ReleaseAllNotes();
    void ResetSamples();
    void OpenPatch();
    void ClosePatch();
    void UnloadSample();
    int LoadSample(void* Data, int sample_id, bool replace);
    int LoadVoiceInfo(int sample_id, int banknr, int instrumentnr, int pan);

    void DebugMode(int level);
    void Equalizer(int bass, int treble);
    void ReverbMode(int mode);
    void ChorusMode(int mode);
    void InitialAttenuation(int value);
    void MainVolume(int volume);

    int AvailableMem();

  private:
    void PlayNote(int voice, int bank, int program, int note);

};

#endif

///////////////////////////////////// eof //////////////////////////////////////
